(**************************************************************************)
(*                                                                        *)
(*      whitelister : a Whitelister Policy Daemon for Postfix             *)
(*      ~~~~~~~~~~~                                                       *)
(*                                                                        *)
(*  Copyright (C) 2005 Polytechnique.org                                  *)
(*  Author : Pierre Habouzit                                              *)
(*                                                                        *)
(*  This program is free software; you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License as published by  *)
(*  the Free Software Foundation; either version 2 of the License, or     *)
(*  (at your option) any later version.                                   *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  along with this program; if not, write to the Free Software           *)
(*  Foundation, Inc.,                                                     *)
(*  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA               *)
(*                                                                        *)
(**************************************************************************)

(* types *)

type config = {
  mutable sock: Unix.sockaddr;
  mutable rbl:  string list;
  mutable rhbl: string list;
}

(* Checker *)

let checker cfg ic oc =
  let pcy = Policy.read ic in
  let ok = (
    try
      Rules.check_rbl cfg.rbl pcy;
      Rules.check_rhbl cfg.rhbl pcy;
      true
    with Rules.Dirty -> false
  ) in
  Printf.fprintf oc "action=%s\n\n" (if ok then "OK" else "DUNNO");
  flush oc

(* Configuration *)

let update_config cfg file =
  let ic  = open_in file in
  (
    try
      while true do
        let line = input_line ic in
        if String.length line > 0 && line.[0] <> '#' then
          match Str.split (Str.regexp "[ :]+") line with
            | [] -> ()
            | ["sock"; s]     -> cfg.sock <- Unix.ADDR_UNIX s
            | ["sock"; ip; p] -> cfg.sock <- Unix.ADDR_INET (Unix.inet_addr_of_string ip, int_of_string p)
            | ["rbl";  h]     -> cfg.rbl  <- h::cfg.rbl
            | ["rhbl"; h]     -> cfg.rhbl <- h::cfg.rhbl
            | _ -> prerr_string (Printf.sprintf "`%s' is not a valid config line\n" line)
      done;
      assert false
    with End_of_file -> ()
  );
  cfg

let find_conf_files () =
  let l = ["/etc/whitelister.conf" ; "whitelister.conf"] in
  let exists f =
    try let _ = Unix.stat f in true
    with _ -> false
  in List.filter exists l

let read_config () =
  match find_conf_files () with
    | [] -> prerr_endline "No config files found"; exit 1
    | l  -> List.fold_left update_config { sock=Unix.ADDR_INET (Unix.inet_addr_of_string "127.0.0.1", 10000); rbl=[] ; rhbl=[] } l

(* Server thingies *)

let cleanup_sock = function
  | Unix.ADDR_INET _ -> ()
  | Unix.ADDR_UNIX u -> (try Unix.unlink u with _ -> ())

(* Main LOOP *)

let _ =
  let cfg = read_config () in
  cleanup_sock cfg.sock;
  Unix.establish_server (checker cfg) cfg.sock

